/* BdfServer - Copyright (c) 2015-2023 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.ui.components;

import fr.exemole.bdfserver.api.BdfServerConstants;
import fr.exemole.bdfserver.api.ui.IncludeUi;
import fr.exemole.bdfserver.api.ui.SpecialIncludeUi;
import fr.exemole.bdfserver.api.ui.SubsetIncludeUi;
import fr.exemole.bdfserver.api.ui.options.InputOption;
import fr.exemole.bdfserver.api.ui.options.ValueOption;
import fr.exemole.bdfserver.tools.ui.options.InputOptionBuilder;
import fr.exemole.bdfserver.tools.ui.options.UiOptionsUtils;
import net.fichotheque.include.ExtendedIncludeKey;
import net.mapeadores.util.annotation.Nullable;
import net.mapeadores.util.attr.AttributeChange;
import net.mapeadores.util.attr.Attributes;
import net.mapeadores.util.attr.AttributesBuilder;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelChangeBuilder;
import net.mapeadores.util.text.Labels;


/**
 *
 * @author Vincent Calame
 */
public class IncludeUiBuilder {

    private final String name;
    private final ExtendedIncludeKey extendedIncludeKey;
    private final AttributesBuilder attributesBuilder;
    private String status = BdfServerConstants.DEFAULT_STATUS;
    private LabelChangeBuilder labelChangeBuilder = new LabelChangeBuilder();
    private boolean withInputOption;
    private boolean withValueOption;
    private InputOption inputOption;
    private ValueOption valueOption;


    private IncludeUiBuilder(String name, ExtendedIncludeKey extendedIncludeKey, @Nullable Attributes initAttributes) {
        this.name = name;
        this.extendedIncludeKey = extendedIncludeKey;
        if (extendedIncludeKey != null) {
            if (extendedIncludeKey.getSubsetKey().isThesaurusSubset()) {
                this.inputOption = UiOptionsUtils.INDEXATION_DEFAULT_INPUTOPTION;
                this.valueOption = UiOptionsUtils.DEFAULT_VALUEOPTION;
                this.withInputOption = true;
                this.withValueOption = true;
            }
        }
        this.attributesBuilder = new AttributesBuilder(initAttributes);
    }

    public IncludeUiBuilder setStatus(String status) {
        if (status != null) {
            this.status = BdfServerConstants.checkValidStatus(status);
        }
        return this;
    }

    public IncludeUiBuilder setInputOption(InputOption inputOption) {
        if ((this.withInputOption) && (inputOption != null)) {
            this.inputOption = inputOption;
        }
        return this;
    }

    public IncludeUiBuilder setInputOption(String inputType, String widthType, int rows) {
        if (this.withInputOption) {
            this.inputOption = InputOptionBuilder.build(inputType, widthType, rows);
        }
        return this;
    }

    public IncludeUiBuilder setValueOption(ValueOption valueOption) {
        if ((this.withValueOption) && (valueOption != null)) {
            this.valueOption = valueOption;
        }
        return this;
    }

    public IncludeUiBuilder putLabel(Label label) {
        labelChangeBuilder.putLabel(label);
        return this;
    }

    public IncludeUiBuilder removeLabel(Lang lang) {
        labelChangeBuilder.putRemovedLang(lang);
        return this;
    }

    public IncludeUiBuilder clearCustomLabels() {
        labelChangeBuilder = new LabelChangeBuilder();
        return this;
    }

    public IncludeUiBuilder changesAttribute(AttributeChange attributeChange) {
        this.attributesBuilder.changeAttributes(attributeChange);
        return this;
    }

    public AttributesBuilder getAttributesBuilder() {
        return attributesBuilder;
    }

    public IncludeUi toIncludeUi() {
        Labels customLabels = labelChangeBuilder.toLabels();
        if (customLabels.isEmpty()) {
            customLabels = null;
        }
        if (extendedIncludeKey == null) {
            return new InternalSpecialIncludeUi(name, status, inputOption, valueOption, customLabels, attributesBuilder.toAttributes());
        } else {
            return new InternalSubsetIncludeUi(name, status, extendedIncludeKey, inputOption, valueOption, customLabels, attributesBuilder.toAttributes());
        }
    }

    public static IncludeUiBuilder initSubset(ExtendedIncludeKey includeKey) {
        if (includeKey == null) {
            throw new IllegalArgumentException("includeKey is null");
        }
        return new IncludeUiBuilder(includeKey.getKeyString(), includeKey, null);
    }

    public static IncludeUiBuilder initSubset(ExtendedIncludeKey includeKey, @Nullable Attributes attributes) {
        if (includeKey == null) {
            throw new IllegalArgumentException("includeKey is null");
        }
        return new IncludeUiBuilder(includeKey.getKeyString(), includeKey, attributes);
    }

    public static SubsetIncludeUi toDefaultSubsetIncludeUi(ExtendedIncludeKey includeKey) {
        if (includeKey == null) {
            throw new IllegalArgumentException("includeKey is null");
        }
        return (SubsetIncludeUi) IncludeUiBuilder.initSubset(includeKey).toIncludeUi();
    }

    public static IncludeUiBuilder initSpecial(String specialIncludeName) {
        if (specialIncludeName == null) {
            throw new IllegalArgumentException("includeKey is null");
        }
        return new IncludeUiBuilder(specialIncludeName, null, null);
    }

    public static IncludeUiBuilder initSpecial(String specialIncludeName, @Nullable Attributes attributes) {
        if (specialIncludeName == null) {
            throw new IllegalArgumentException("includeKey is null");
        }
        return new IncludeUiBuilder(specialIncludeName, null, attributes);
    }

    public static SpecialIncludeUi toDefaultSpecialIncludeUi(String specialIncludeName) {
        if (specialIncludeName == null) {
            throw new IllegalArgumentException("includeKey is null");
        }
        return (SpecialIncludeUi) IncludeUiBuilder.initSpecial(specialIncludeName).toIncludeUi();
    }

    public static IncludeUiBuilder derive(IncludeUi includeUi) {
        ExtendedIncludeKey extendedIncludeKey = null;
        if (includeUi instanceof SubsetIncludeUi) {
            extendedIncludeKey = ((SubsetIncludeUi) includeUi).getExtendedIncludeKey();
        }
        IncludeUiBuilder builder = new IncludeUiBuilder(includeUi.getName(), extendedIncludeKey, includeUi.getAttributes())
                .setStatus(includeUi.getStatus())
                .setInputOption(includeUi.getInputOption())
                .setValueOption(includeUi.getValueOption());
        Labels customLabels = includeUi.getCustomLabels();
        if (customLabels != null) {
            for (Label label : customLabels) {
                builder.putLabel(label);
            }
        }
        return builder;
    }


    private static class InternalSpecialIncludeUi implements SpecialIncludeUi {

        private final String name;
        private final String status;
        private final InputOption inputOption;
        private final ValueOption valueOption;
        private final Labels customLabels;
        private final Attributes attributes;

        private InternalSpecialIncludeUi(String name, String status, InputOption inputOption, ValueOption valueOption, Labels customLabels, Attributes attributes) {
            this.name = name;
            this.status = status;
            this.inputOption = inputOption;
            this.valueOption = valueOption;
            this.customLabels = customLabels;
            this.attributes = attributes;
        }

        @Override
        public String getName() {
            return name;
        }

        @Override
        public String getStatus() {
            return status;
        }

        @Override
        public InputOption getInputOption() {
            return inputOption;
        }

        @Override
        public ValueOption getValueOption() {
            return valueOption;
        }

        @Override
        public Labels getCustomLabels() {
            return customLabels;
        }

        @Override
        public Attributes getAttributes() {
            return attributes;
        }

    }


    private static class InternalSubsetIncludeUi implements SubsetIncludeUi {

        private final String name;
        private final ExtendedIncludeKey extendedIncludeKey;
        private final String status;
        private final InputOption inputOption;
        private final ValueOption valueOption;
        private final Labels customLabels;
        private final Attributes attributes;

        private InternalSubsetIncludeUi(String name, String status, ExtendedIncludeKey extendedIncludeKey, InputOption inputOption, ValueOption valueOption, Labels customLabels, Attributes attributes) {
            this.name = name;
            this.status = status;
            this.extendedIncludeKey = extendedIncludeKey;
            this.inputOption = inputOption;
            this.valueOption = valueOption;
            this.customLabels = customLabels;
            this.attributes = attributes;
        }

        @Override
        public String getName() {
            return name;
        }

        @Override
        public String getStatus() {
            return status;
        }

        @Override
        public ExtendedIncludeKey getExtendedIncludeKey() {
            return extendedIncludeKey;
        }

        @Override
        public InputOption getInputOption() {
            return inputOption;
        }

        @Override
        public ValueOption getValueOption() {
            return valueOption;
        }

        @Override
        public Labels getCustomLabels() {
            return customLabels;
        }

        @Override
        public Attributes getAttributes() {
            return attributes;
        }

    }

}
