/* FichothequeLib_API - Copyright (c) 2011-2018 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.corpus.metadata;

import java.util.List;
import net.fichotheque.SubsetKey;
import net.mapeadores.util.annotation.Nullable;
import net.mapeadores.util.localisation.Langs;
import net.mapeadores.util.money.Currencies;


/**
 * Interface décrivant les options des champs de fiche reconnues par le coeur de
 * BDF. D'autres options peuvent être définies par une extension et seront
 * accessibles via CorpusField.getStringOption().
 *
 * @author Vincent Calame
 */
public interface FieldOptions {

    /**
     * Retourne la sphère par défaut pour le traitement des champs de type «
     * Personne ». Peut être nul (et dans ce cas la sphère par défaut sera celle
     * du rédacteur intervenant sur la fiche). SubsetKey doit désigner une
     * sphère et non une autre collecion.
     *
     * @return un objet SubsetKey indentifiant la sphère par défaut ou null
     */
    public SubsetKey getDefaultSphereKey();

    /**
     * Retourne la liste des monnaies acceptées par le champ. Peut être nul mais
     * pas de longueur nul.
     *
     * @return une instance de Currencies listant les monnaies acceptées ou null
     */
    @Nullable
    public Currencies getCurrencies();

    /**
     * Retourne la liste des champs permettant de construire une adresse. Cette
     * options est utilisée pour le géocodage. L'adresse de géocodage sera
     * construite en concaténant les valeurs des champs indiqués par l'instance
     * de CorpusFieldList. Peut être nul.
     *
     * @return une instance de List<CorpusField> listant les champs pour
     * construire une adresse ou null
     */
    public List<CorpusField> getAddressFieldList();

    /**
     * Retourne l'affichage privilégié pour les éléments d'un champ Information.
     * Deux valeurs sont possibles :
     * <ul>
     * <li>FieldOptionsConstants.INLINE_DISPLAY : les éléments sont à la suite
     * (séparés par un point-virgule)
     * <li>FieldOptionsConstants.BLOCK_DISPLAY : les éléments sont séparés par
     * un saut de ligne
     * </ul>
     * <p>
     * Retourne nul si et seulement le champ en question n'est un champ
     * Information. Autrement dit, cette valeur n'est jamais nul s'il s'agit
     * d'un champ Information.
     *
     * @return le type d'affichage priviligié d'un champ Information ou nul pour
     * une autre catégorie de champ
     */
    public Short getInformationDisplay();

    /**
     * Indique si le champ doit être décomposé en sous-champ dans lors de la
     * saisie. Cette option n'est pertinente que pour les Propriétés dont le
     * type possède des sous-champs (par exemple, un propriété de type «
     * Geopoint » possède deux sous-champs : la latitude et la longitude).
     *
     * @return true si la décomposition en sous-champ doit être utilisée, false
     * sinon
     */
    public boolean getSubfieldDisplay();

    /**
     * Retourne l'étendue des langues disponibles pour les champs traitant des
     * langues. Les trois valeurs possibles sont:
     * <ul>
     * <li>FieldOptionsConstants.ALL_SCOPE : toutes les langues sont disponibles
     * <li>FieldOptionsConstants.CONFIG_SCOPE : les langues disponibles sont
     * celles de la configuration globale
     * <li>FieldOptionsConstants.LIST_SCOPE : les langues disponibles sont
     * définies par la liste retournée par getLangs() (sa valeur ne doit
     * impérativement pas être nulle)
     * </ul>
     * <p>
     * Cette méthode doit retourner une valeur pour le champ de la langue de la
     * fiche (FieldKey.LANG) et pour tous les champs Information ou Propriété de
     * type langue et nul pour tous les autres champs.
     * <p>
     * La valeur par défaut est différente suivant les champs : pour le champ de
     * la langue de la fiche (FieldKey.LANG), elle est vaut
     * FieldOptionsConstants.CONFIG_SCOPE et elle vaut
     * FieldOptionsConstants.ALL_SCOPE pour tous les autres champs de type
     * langue.
     *
     * @return l'indication de l'étendue des langues disponibles ou nul si le
     * champ n'est pas concerné
     */
    @Nullable
    public Short getLangScope();

    @Nullable
    public Langs getLangs();

}
